function d = lbfgsinverse_evalBkd(gradfxk, k, m, rho, s, y)
% d = lbfgsinverse_evalBkd(gradfxk, k, m, rho, s, y)
%
% Calculates the search direction dk = Bk * gradf(xk)
% without explicitly forming Bk
%
% INPUT:  gradfxk --> current gradient of f at xk
%               k --> current iteration number
%               m --> history size
%             rho --> cell array of all previous rho values
%               s --> cell array of all previous sk vectors 
%               y --> cell array of all previous yk vectors
%
% OUTPUT:       d --> product B_k * gradf(xk)
%
% Implementation according to Nocedal & Wright, 2006
% Algorithm 9.1 (L-BFSG two-loop recursion)
%

% determine dimension
% n = length(gradfxk);

% restrict memory size to current available history
m = min(k-1, m);

% use default initial inverse Hessian approximation
Bk0 = ( s{k-1}.' * y{k-1} )  /  ( y{k-1}.' * y{k-1} );
% note: BK0 is a scalar, not a matrix here!

% FOR COMPARISON WITH FULL-MEMORY BFGS: Deactivate scaling
% Bk0 = 1; 

% prepare helper vector
alpha = zeros(m,1);

% algorithm 9.1
q = gradfxk;
for i = k-1:-1:k-m
   ii = i + (m-k) + 1;  % index shift for alpha
   alpha(ii) = rho{i} * s{i}.' * q;
   q = q - alpha(ii) * y{i};
end
d = Bk0 * q;   % note: Bk0 is a scaling *factor* here!
for i = k-m:+1:k-1
   ii = i + (m-k) + 1;  % index shift for alpha
   beta = rho{i} * y{i}.' * d;
   d = d + s{i} * (alpha(ii) - beta);
end


% For Comparison: Build the matrix Bk explicitly
% Bk = lbfgsinverse_buildBk(k, m, Bk0, s, y); 
% dk2 = Bk * gradfxk; assignin('base','Bktrue',Bk);
% disp(eig(Bk)); keyboard
% if (norm(d-dk2)>1.0d-15); keyboard; end



end
