% set problem parameters
set_alpha = [1, 10, 100];        % non-linearity of ext. Rosenbrock
set_m     = [1, 5, 10, 20, inf]; % l-bfgs memory
set_n     = [100, 1000, 10000];  % problem dimension

% single run shortcut
set_alpha = 100; set_m = 10; set_n = 10000;

% flag to compare with full memory inverse BFGS
compareToFullInverseBFGS = false;

% set solver parameters
eta    = 0.9;      % powell wolfe
gamma  = 1.0d-4;   % powell wolfe
tol    = 1.0d-6;   % stationarity tolerance
maxit  = 10000;    % maximum number of iterations
output = false;    % display output info on every iteration


% cycle through problems
for alpha = set_alpha
   fprintf('\nNonlinearity factor ALPHA = %g\n\n', alpha)
   
   % setup extended Rosenbrock functions
   f     = @(x) extRosenbrock(x,alpha);
   gradf = @(x) extRosenbrockGradient(x,alpha);

   for n = set_n
     
      for m = set_m
      
         % random initial guess
         rng(1234);       % fixed random seed for comparison and testing
         %rng('shuffle'); % randomized
         mean = 0.0;
         sd   = 20.0;
         x0   = random('normal', mean, sd, [n 1]);

         % reset function counters
         extRosenbrock('reset');
         extRosenbrockGradient('reset');

         % Limited Memory Inverse BFGS
         ticID = tic();  
         [X,info] = lbfgsinverse(f, gradf, m, x0, tol, maxit, gamma, eta, output);
         time  = toc(ticID);

         % count function evaluations
         nf       = extRosenbrock('count');
         ngradf   = extRosenbrockGradient('count');

         % compare to solution
         sol      = ones(n,1);   % solution
         soldiff  = norm(X(:,end)-sol,2);
         fprintf('n = %-7d   m = %-4d   converged = %-5s   #iter = %5d   #f = %-5d   #gradf = %-5d   time=%9.4fs   ||error||=%14.8g\n', ...
            n, m, mat2str(~info), size(X,2), nf, ngradf, time, soldiff);

         % Comparison: Standard Inverse BFGS
         if compareToFullInverseBFGS
            B0 = eye(n);
            extRosenbrock('reset');
            extRosenbrockGradient('reset');
            ticID = tic();
            [X,info] = bfgsinverse(f, gradf, B0, x0, tol, maxit, gamma, eta, output);
            time     = toc(ticID);
            nf       = extRosenbrock('count');
            ngradf   = extRosenbrockGradient('count');
            soldiff  = norm(X(:,end)-sol,2);
            fprintf('n = %-7d   m = %-4d   converged = %-5s   #iter = %5d   #f = %-5d   #gradf = %-5d   time=%9.4fs   ||error||=%14.8g\n', ...
               n, inf, mat2str(~info), size(X,2), nf, ngradf, time, soldiff);
         end

      end
   end
end

% finito
return



